#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.6.1
 Author:         Matwachich

 Script Function:
	Fonctions pour crer des icones cliquables  3 positions:
	(Normale, Survol, Clique)
	
 Versions:
	- Version 1: 17-03-2011
		_3StatIcon_Create
		_3StatIcon_ChangeIcons
		_3StatIcon_Delete
		
	- Version 2: 18-03-2011
		+ _3StatIcon_ResetIcon
		+ Documentation des fonctions (Anglais)
		- Peut utiliser des fichiers de ressources (dll, exe)
	
 Notes:
	- J'utilise l'UDF Icons.au3 car le rendu est bien meilleur
	
 Todo:

#ce ----------------------------------------------------------------------------

#include <perso\Icons.au3>
#include <perso\GuiCtrlOnHover.au3>
#include <Array.au3>

Global $__3IB_Array
__3StatIcon_InitArray()

; $sIco1 = Normal stat icon file	($__3IB_Array[$i][1])
; $iIndex1 = Normal stat icon index ($__3IB_Array[$i][2])
; $sIco2 = Hover stat icon file		($__3IB_Array[$i][3])
; $iIndex2 = Hover stat icon index 	($__3IB_Array[$i][4])
; $sIco3 = Click stat icon file		($__3IB_Array[$i][5])
; $iIndex3 = Click stat icon index 	($__3IB_Array[$i][6])

; # FUNCTION # ================================================================================================
; Name...........: _3StatIcon_Create
; Description....: Create an Icon/button with 3 positions (normal, hover, clicked)
; Parameters.....: 	$sIco1 = path to the file of normal stat (.ico, .dll, .exe)
;					$iIndex1 = Index of the ressource file of normal stat (if $sIco1 is a dll or exe file)
;						Otherwise (if $sIco1 is and .ico file), it must be set to 0
;					*PS*: The same thing for parameters $sIco2, $iIndex2 and $sIco3, $iIndex3, they are
;						the hover stat and clicked stat in this order.
;						If they are set to default (or omited), then the same image as $sIco1 will be used
;					$iXpos, $iYpos = Standard X and Y pos of the control
;					$iWidth, $iHeight = Width and Height of the control
;					$iStyle, $iExtStyle = Style ane Extended Style of the GuiCtrlIcon (Optional)
; Return values..: The handle of the control (GuiCtrlIcon)
; Author.........: Matwachich
; Remarks........:   
; ===========================================================================================================
Func _3StatIcon_Create($sIco1, $iIndex1, $iXpos, $iYpos, $iWidth, $iHeight, _
						$sIco2 = Default, $iIndex2 = 0, $sIco3 = Default, $iIndex3 = 0, _
						$iStyle = Default, $iExtStyle = Default)
								
	Local $hIco = GuiCtrlCreateIcon("", "", $iXpos, $iYpos, $iWidth, $iHeight, $iStyle, $iExtStyle)
	_GUICtrl_OnHoverRegister($hIco, "__3StatIcon_OnHoverFunction", "__3StatIcon_OnHoverFunction", _
									"__3StatIcon_OnClickFunction", "__3StatIcon_OnClickFunction", 0)
	_SetIcon($hIco, $sIco1, $iIndex1, $iWidth, $iHeight)
	
	If $sIco2 = Default Then $sIco2 = $sIco1
	If $sIco3 = Default Then $sIco3 = $sIco1
	__3StatIcon_ArrayAdd($hIco, $sIco1, $iIndex1, $sIco2, $iIndex2, $sIco3, $iIndex3)
	
	Return $hIco
EndFunc

; # FUNCTION # ================================================================================================
; Name...........: _3StatIcon_ResetIcon
; Description....: Resets an 3stats icon (or all the controls) to the normal stat
; Parameters.....: 	$hCtrl = Handle of the control to reset (as returned by _3StatIcon_Create)
;						If = -1, all the created controls will be reset to normal stat
; Return values..: 1
; Author.........: Matwachich
; Remarks........:   
; ===========================================================================================================
Func _3StatIcon_ResetIcon($hCtrl = -1)
	If $hCtrl = -1 Then
		For $i = 1 To $__3IB_Array[0][0]
			_SetIcon($__3IB_Array[$i][0], $__3IB_Array[$i][1], $__3IB_Array[$i][2])
		Next
	Else
		__3StatIcon_OnHoverFunction($hCtrl, 2, 0) ; leave function
	EndIf
	Return 1
EndFunc

; # FUNCTION # ================================================================================================
; Name...........: _3StatIcon_ChangeIcons
; Description....: Changes some/all of the 3stat icons of a control
; Parameters.....: 	$hCtrl = Handle of the control to alter (as returned by _3StatIcon_Create)
;					$sIco1, $iIndex1 ... $sIco3, $iIndex3 = The same as _3StatIcon_Create, but here:
;						If an parameter is omited, the it is not changed
; Return values..: 	Succes = 1
;					Failed = 0 (hCtrl is not a valide 3Stat button control)
; Author.........: Matwachich
; Remarks........:   
; ===========================================================================================================
Func _3StatIcon_ChangeIcons($hCtrl, $sIco1 = Default, $iIndex1 = Default, _
									$sIco2 = Default, $iIndex2 = Default, _
									$sIco3 = Default, $iIndex3 = Default)
									
	Local $iCtrlIndex = __3StatIcon_hCtrlToIndex($hCtrl)
	If $iCtrlIndex = -1 Then Return 0
	If $sIco1 <> Default Then $__3IB_Array[$iCtrlIndex][1] = $sIco1
		If $iIndex1 <> Default Then $__3IB_Array[$iCtrlIndex][2] = $iIndex1
	If $sIco2 <> Default Then $__3IB_Array[$iCtrlIndex][3] = $sIco2
		If $iIndex2 <> Default Then $__3IB_Array[$iCtrlIndex][4] = $iIndex2
	If $sIco3 <> Default Then $__3IB_Array[$iCtrlIndex][5] = $sIco3
		If $iIndex3 <> Default Then $__3IB_Array[$iCtrlIndex][6] = $iIndex3
	Return 1
EndFunc

; # FUNCTION # ================================================================================================
; Name...........: _3StatIcon_Delete
; Description....: Deletes an 3Stat Button
; Parameters.....: $hCtrl = Handle of the control to delete (as returned by _3StatIcon_Create)
; Return values..: Succes = 1, Failure = 0
; Author.........: Matwachich
; Remarks........:   
; ===========================================================================================================
Func _3StatIcon_Delete($hCtrl)
	__3StatIcon_ArrayDel($hCtrl)
	Return GuiCtrlDelete($hCtrl)
EndFunc

; ##############################################################
; ################## Internal use only #########################
; ##############################################################

Func __3StatIcon_OnClickFunction($hCtrl, $iClickMode)
	Local $iCtrlIndex = __3StatIcon_hCtrlToIndex($hCtrl)
	If $iCtrlIndex = -1 Then Return 0
	
	Switch $iClickMode
		Case 1 ; pressed
			_SetIcon($hCtrl, $__3IB_Array[$iCtrlIndex][5], $__3IB_Array[$iCtrlIndex][6])
		Case 2 ; released
			_SetIcon($hCtrl, $__3IB_Array[$iCtrlIndex][3], $__3IB_Array[$iCtrlIndex][4])
	EndSwitch
EndFunc

Func __3StatIcon_OnHoverFunction($hCtrl, $iHoverMode, $hWin)
	Local $iCtrlIndex = __3StatIcon_hCtrlToIndex($hCtrl)
	If $iCtrlIndex = -1 Then Return 0
		
	Switch $iHoverMode
		Case 1 ; hover
			_SetIcon($hCtrl, $__3IB_Array[$iCtrlIndex][3], $__3IB_Array[$iCtrlIndex][4])
		Case 2 ; leave
			_SetIcon($hCtrl, $__3IB_Array[$iCtrlIndex][1], $__3IB_Array[$iCtrlIndex][2])
	EndSwitch
EndFunc

Func __3StatIcon_ArrayAdd($hCtrl, $sIco1, $iIndex1, $sIco2, $iIndex2, $sIco3, $iIndex3)
	Local $iNbr = $__3IB_Array[0][0]
	Redim $__3IB_Array[$iNbr + 2][7]
	$__3IB_Array[$iNbr + 1][0] = $hCtrl
	$__3IB_Array[$iNbr + 1][1] = $sIco1
	$__3IB_Array[$iNbr + 1][2] = $iIndex1
	$__3IB_Array[$iNbr + 1][3] = $sIco2
	$__3IB_Array[$iNbr + 1][4] = $iIndex2
	$__3IB_Array[$iNbr + 1][5] = $sIco3
	$__3IB_Array[$iNbr + 1][6] = $iIndex3
	$__3IB_Array[0][0] = $iNbr + 1
EndFunc

Func __3StatIcon_ArrayDel($hCtrl)
	For $i = 1 To $__3IB_Array[0][0]
		If $__3IB_Array[$i][0] = $hCtrl Then
			_ArrayDelete($__3IB_Array, $i)
			If Not IsArray($__3IB_Array) Then __3StatIcon_InitArray()
			Return 1
		EndIf
	Next
	Return 0
EndFunc

Func __3StatIcon_hCtrlToIndex($hCtrl)
	Local $iCtrlIndex = -1
	For $i = 1 To $__3IB_Array[0][0]
		If $__3IB_Array[$i][0] = $hCtrl Then
			$iCtrlIndex = $i
			ExitLoop
		EndIf
	Next
	Return $iCtrlIndex
EndFunc

Func __3StatIcon_InitArray()
	Global $__3IB_Array[1][7]
	$__3IB_Array[0][0] = 0
EndFunc
